//
//  iAdBasicViewController.m
//  iAdBasic
//
//  Created by Tomonari Shimada.
//  Copyright 2010 Blueair Inc. All rights reserved.
//

#import "iAdBasicViewController.h"
// ネットワークの判定クラスのヘッダファイル読み込み
#import "Reachability.h"

@interface iAdBasicViewController (privatemethods)
// メソッドの定義
- (void)createAdBannerView;
- (void)showAdBanner;
- (void)hideAdBanner;
- (void)changeAdBannerOrientation:(UIInterfaceOrientation)toInterfaceOrientation;
- (BOOL)checkInternet;
@end

@implementation iAdBasicViewController

@synthesize adBannerView;
@synthesize internetReach;

#pragma mark -
#pragma mark Initialization

// ビューがロードされたら呼ばれる
- (void)viewDidLoad
{
    [super viewDidLoad];
    
    // ネットワークの状態を取得するクラスをインスタンス化
    internetReach = [[Reachability reachabilityForInternetConnection] retain];
    // ネットワークの状態を取得するメソッドをスタートさせる
	[internetReach startNotifier];
    
    // ネットワークに繋がっているか確認
    BOOL reachable = [self checkInternet];
	
    // ネットワークに繋がっていて、まだバナーが作成されていなかったらバナーを作成するメソッドを呼ぶ
    if (reachable && (self.adBannerView == nil)) {
        [self createAdBannerView];
    }
    
    // ネットワークの状態が変化したら呼ばれるメソッドを設定
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(reachabilityChanged:)
                                                 name:kReachabilityChangedNotification 
                                               object:nil];
}

#pragma mark -
#pragma mark iAd Banner View

// バナーを作成
- (void)createAdBannerView
{
    // デバイスの向きで表示するバナーのサイズを選択
    NSString *adContentSize = UIInterfaceOrientationIsPortrait(self.interfaceOrientation) ? 
    ADBannerContentSizeIdentifier320x50 : ADBannerContentSizeIdentifier480x32;
    
    // バナーをインスタンス化
    ADBannerView *adView = [[ADBannerView alloc] initWithFrame:CGRectZero];
    adView.delegate = self;
    
    // バナーのサイズを設定
    adView.requiredContentSizeIdentifiers = [NSSet setWithObjects:ADBannerContentSizeIdentifier320x50, 
                                             ADBannerContentSizeIdentifier480x32, nil];
    
    // 表示する広告のサイズを設定
    adView.currentContentSizeIdentifier = adContentSize;
    adView.autoresizingMask = UIViewAutoresizingFlexibleBottomMargin | UIViewAutoresizingFlexibleRightMargin;
    
    // バナーの表示座標をバナーの高さ分引く
    CGRect bannerFrame =adView.frame;
    bannerFrame.origin.y = - bannerFrame.size.height;
    adView.frame = bannerFrame;
    
    self.adBannerView = adView;
    
    // バナーをビューに表示する
    [self.view addSubview:adView];
    [adView release];
}

// バナーを表示
- (void)showAdBanner
{
    // バナーの表示座標を0にセット
	CGRect bannerFrame = self.adBannerView.frame;
    bannerFrame.origin.y = 0; 
	
    // バナーをアニメーションしながら表示
	[UIView beginAnimations:nil context:NULL];
	self.adBannerView.frame = bannerFrame;
	[UIView commitAnimations];
}

// バナーを非表示
- (void)hideAdBanner
{	
    // バナーの表示座標をバナーの高さ分引く
	CGRect bannerFrame = self.adBannerView.frame;
	bannerFrame.origin.y = - bannerFrame.size.height;
	
    // バナーをアニメーションしながら非表示
	[UIView beginAnimations:nil context:NULL];
	self.adBannerView.frame = bannerFrame;
	[UIView commitAnimations];
}

// デバイスの向きに応じてバナーのサイズを変更するメソッド
- (void)changeAdBannerOrientation:(UIInterfaceOrientation)toInterfaceOrientation
{
    // デバイスが横のとき
	if (UIInterfaceOrientationIsLandscape(toInterfaceOrientation)) {
		self.adBannerView.currentContentSizeIdentifier = ADBannerContentSizeIdentifier480x32;
    // デバイスが縦のとき
	} else {
        self.adBannerView.currentContentSizeIdentifier = ADBannerContentSizeIdentifier320x50;
	}
}

#pragma mark -
#pragma mark ADBannerViewDelegate methods

// 広告をロードしたとき呼ばれる
- (void)bannerViewDidLoadAd:(ADBannerView *)banner
{
	[self showAdBanner]; // バナーを表示する
}

// 広告のロードに失敗したとき呼ばれる
- (void)bannerView:(ADBannerView *)banner didFailToReceiveAdWithError:(NSError *)error
{
	[self hideAdBanner]; // バナーを非表示にする
    // iAdから広告が提供されなかった場合は別の広告ネットワークのバナーを表示する
}

// バナーがタッチされたとき呼ばれる
-(BOOL)bannerViewActionShouldBegin:(ADBannerView *)banner willLeaveApplication:(BOOL)willLeave
{
    NSLog(@"広告の表示・アクションが始まります");
    // 広告が表示される前にメインの処理や表示を一時停止する必要があります
    return YES;
}

// 広告の表示・アクションが終了したら呼ばれる
-(void)bannerViewActionDidFinish:(ADBannerView *)banner
{
    NSLog(@"広告の表示・アクションが終了しました");
}

#pragma mark -
#pragma mark Device orientation

// デバイスの回転に対応するか設定
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return YES;
}

// デバイスが回転を始めたときに実行されるメソッド
- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration
{
    // デバイスの向きに応じてバナーのサイズを変更するメソッドを呼び出す
    if (self.adBannerView) {
		[self changeAdBannerOrientation:toInterfaceOrientation];
	}
}

#pragma mark -
#pragma mark Network check

// ネットワークに接続出来るか確認するメソッド
- (BOOL)checkInternet
{
    NetworkStatus netStatus = [internetReach currentReachabilityStatus];
    BOOL connectionRequired = [internetReach connectionRequired];
    
    // ネットワークに繋がっているのであれば「YES」
    if ((netStatus == ReachableViaWiFi) || ((netStatus == ReachableViaWWAN) && !connectionRequired)) {
        return YES;
    }
    
    // ネットワークに繋がっていないのであれば「NO」
    return NO;
}

// ネットワークに変化があった際に呼ばれる
- (void)reachabilityChanged:(NSNotification *)note
{
    // ネットワークに繋がっているか確認
    BOOL reachable = [self checkInternet];
	
    // ネットワークに繋がっていて、まだバナーが作成されていなかったらバナーを作成するメソッドを呼ぶ
    if (reachable && (self.adBannerView == nil)) {
        [self createAdBannerView];
    }
    
    // ネットワークに繋がっていない状態で、バナーが表示されていたら非表示にする
    if (!reachable && self.adBannerView) {
        [self hideAdBanner];
        adBannerView.delegate = nil;
        self.adBannerView = nil;
    }
}

#pragma mark -
#pragma mark Memory management

// メモリ警告のとき呼ばれる
- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

// ビューがアンロードしたら呼ばれる
- (void)viewDidUnload
{
    adBannerView.delegate = nil;
    self.adBannerView = nil;
}

// メモリ解放処理
- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [internetReach release];
    adBannerView.delegate = nil;
    self.adBannerView = nil;
    [adBannerView release];
    [super dealloc];
}

@end
